/*!\file
\brief The main file of watercolor2rgba.

A command-line tool that equips scanned images with transparency 
information.

watercolor2rgba is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

watercolor2rgba is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <getopt.h>
#include <Imlib2.h>
#include <ctype.h>
#include "gettext.h"
#define _(String) gettext (String)
#include "configure.h"

  char* helpmessage; 

/*! \brief This function is called on exit of the main program.

*/
void exitfunc()
{
  free(helpmessage);
}
/*! The main dunction */
int main(int argc, char **argv)
{
  int mintransparencyoffset=10;
  int maxtransparencyoffset=10;
  /* an image handle */
  Imlib_Image WhiteBackground;
  Imlib_Image BlackBackgnd;
  Imlib_Image Dest;

  helpmessage = strdup(_("Compares two version of the same image:\n"
      "One version scanned with a white sheet of paper behind it\n"
      "and one scanned with a black sheet of paper put behind it instead:\n"
      "The result will be the original image with added transparency information\n\n"
      "Usage:\n\n"
			 "watercolor2rgba InputfileScannedWithWhiteBackground.png InputfileScannedWithBlackBackground.png Outputfile.png\n"));
 
  atexit(exitfunc);
 {
    static struct option long_options[] =
      {
	{"help",     no_argument,       0, 'h'},
	{"version",  no_argument,       0, 'v'},
	{"min-threshold-offset",     required_argument,       0, 'm'},
	{"max-threshold-offset",     required_argument,       0, 'M'},
	{0, 0, 0, 0}
      };
  
    int index;
    int c;
  
    opterr = 0;
  
    while ((c = getopt_long (argc, argv, "hvm:M:",long_options,&index)) != -1)
      switch (c)
	{
	case'v':
	  fprintf(stderr,_("%s v. %s\n"),argv[0],PACKAGE_VERSION);
	  fprintf(stderr,_("Copyright (C) 2010-2011 Gunter Koenigsmann.\n"));
	  fprintf(stderr,_("License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"));
	  fprintf(stderr,_("This is free software: you are free to change and redistribute it.\n"));
	  fprintf(stderr,_("There is NO WARRANTY, to the extent permitted by law.\n"));
	  exit(0);
	  break;
	case 'h':
	  {
	    fprintf (stderr, "%s",helpmessage);
	    exit(0);
	  }
	  break;
	case 'm':
	  errno=0;
	  mintransparencyoffset=strtol(optarg,NULL,10);
	  if(errno)
	    {
	      fprintf(stderr,_("Cannot interpret the min transparency offset factor\n"));
	      exit(-1);
	    }
	  break;
	case 'M':
	  errno=0;
	  mintransparencyoffset=strtol(optarg,NULL,10);
	  if(errno)
	    {
	      fprintf(stderr,_("Cannot interpret the max transparency offset factor\n"));
	      exit(-1);
	    }
	  break;
	case '?':
	  if (optopt == 'c')
	    {
	      fprintf (stderr, _("Option -%c requires an argument.\n"), optopt);
	      exit(-1);
	    }
	  else if (isprint (optopt))
	    {
	      fprintf (stderr, _("Unknown option `-%c'.\n"), optopt);
	      exit(-1);
	    }
	  else
	    {
	      fprintf (stderr,
		       _("Unknown option character `\\x%x'.\n"),
		       optopt);
	      exit(-1);
	    }
	  return 1;
	default:
	  abort ();
	}
  }

  
  /* if the user has provided < 3 arguments after the command - exit */
  if (argc - optind != 3)
    {
      fprintf(stderr,"%s",helpmessage);
      exit(1);
    }
  /* load the image */
  printf(_("Loading the image scanned with a white background\n"));
  WhiteBackground = imlib_load_image(argv[optind]);
  /* if the load was successful */
  if (WhiteBackground)
    {      
      printf(_("Loading the image scanned with a black background\n"));
      BlackBackgnd=imlib_load_image(argv[optind+1]);
      if(BlackBackgnd)
	{
	  long int width,height,x,y;
	  int i,min_transparency=0,max_transparency=0;
	  long int histogram[256];
	    
	  int min_negative_transparency=0;

	  imlib_context_set_image(WhiteBackground);
	  width=imlib_image_get_width();
	  height=imlib_image_get_height();
	  // Paranoia: What if both images differ in size?
	  imlib_context_set_image(BlackBackgnd);
	  if(width!=imlib_image_get_width()||height!=imlib_image_get_height())
	    {
	      fprintf(stderr,_("Images differ in size! Aborting.\n"));
	      exit(-1);
	    }
	  
	  Dest=imlib_create_image(width,height);
	  imlib_context_set_image(Dest);
	  imlib_image_set_has_alpha(1);
	  imlib_image_clear_color(0,0,0,0);

	  {
	    
	    printf(_("Calculating a transparency histogram of the image\n"));

	    memset(&histogram,0,sizeof(histogram));
	    
	    for(y=0;y<width;y++)
	      for(x=0;x<width;x++)
		{
		  int difference;
		  Imlib_Color Pixel_WhiteBackgnd;
		  Imlib_Color Pixel_BlackBackgnd;
		  imlib_context_set_image(WhiteBackground);
		  imlib_image_query_pixel(x,y,&Pixel_WhiteBackgnd);
		  imlib_context_set_image(BlackBackgnd);
		  imlib_image_query_pixel(x,y,&Pixel_BlackBackgnd);

		  difference=Pixel_WhiteBackgnd.red  -Pixel_BlackBackgnd.red+
		    Pixel_WhiteBackgnd.green-Pixel_BlackBackgnd.green+
		    Pixel_WhiteBackgnd.blue -Pixel_BlackBackgnd.blue;
		  if((difference>=0)&&(difference<256))
		    histogram[difference]++;

		  if(difference<min_negative_transparency)
		    min_negative_transparency=difference;
		}

	    if(min_negative_transparency<-20)
	      printf(_("Warning: Alignment of the images might be bad: Minimum Transparency=%i\n"),
		     min_negative_transparency);

	    printf(_("Trying to find the transparency span: "));
	    long int peaksteepness_last=0;
	    for(i=10;i<245;i++)
	      {
		int peaksteepness=-histogram[i-1]-histogram[i-2]-histogram[i-3]+
		  histogram[i+1]+histogram[i+2]+histogram[i+3];
		if((peaksteepness<0)&&(peaksteepness_last>0))
		  {
		    if(min_transparency==0) min_transparency=i;
		    max_transparency=i;
		  }
		peaksteepness_last = peaksteepness;
	      }
	    
	    // Allow some noise margin for "fully transparent" and "fully oblique".
	    min_transparency+=(max_transparency-min_transparency)*mintransparencyoffset/100;
	    max_transparency-=(max_transparency-min_transparency)*maxtransparencyoffset/100;
	    
	    if(min_transparency<0)min_transparency=0;
	    if(max_transparency>255)max_transparency=255;

	    printf(_("Min=%i, Max=%i\n"),
		   min_transparency,max_transparency);
	    
	    if (max_transparency<=min_transparency)
	      {
		fprintf(stderr,_("Didn't succeed analyzing the transparency histogram.\n"));
		exit(-1);
	      }
	    
	    max_transparency = max_transparency-min_transparency;
	    
	    printf(_("Now calculating the image.\n"));
	    for(y=0;y<width;y++)
	      for(x=0;x<width;x++)
		{
		  int difference;
		  Imlib_Color Pixel_WhiteBackgnd;
		  Imlib_Color Pixel_BlackBackgnd;
		  Imlib_Color NewPixel;
		  imlib_context_set_image(BlackBackgnd);
		  imlib_image_query_pixel(x,y,&Pixel_BlackBackgnd);
		  imlib_context_set_image(WhiteBackground);
		  imlib_image_query_pixel(x,y,&Pixel_WhiteBackgnd);

		  // Calculate how big a difference the background makes.
		  difference=Pixel_WhiteBackgnd.red  -Pixel_BlackBackgnd.red+
		    Pixel_WhiteBackgnd.green-Pixel_BlackBackgnd.green+
		    Pixel_WhiteBackgnd.blue -Pixel_BlackBackgnd.blue-min_transparency;

		  
		  if(difference>max_transparency)difference=max_transparency;

		  if(difference<min_transparency)difference=min_transparency;

		  NewPixel.alpha=255*(max_transparency-difference)/(max_transparency-min_transparency);
		  NewPixel.red=  Pixel_WhiteBackgnd.red  *(NewPixel.alpha)/255;
		  NewPixel.green=Pixel_WhiteBackgnd.green*(NewPixel.alpha)/255;
		  NewPixel.blue= Pixel_WhiteBackgnd.blue *(NewPixel.alpha)/255;

		  imlib_context_set_color(NewPixel.red,NewPixel.green,NewPixel.blue,NewPixel.alpha);
		  imlib_context_set_image(Dest);
		  imlib_image_draw_pixel(x,y,1);
		}
	  }
	  imlib_context_set_image(BlackBackgnd);
	  imlib_free_image();
	  imlib_context_set_image(WhiteBackground);
	  imlib_free_image();

	  printf(_("Saving the image.\n"));
	  
	  imlib_context_set_image(Dest);
	  char *tmp;
	  /* argument - i.e. .png = png, .tif = tiff etc. */
	  tmp = strrchr(argv[optind+2], '.');
	  if(tmp)
	    imlib_image_set_format(tmp + 1);
	  /* save the image */
	  imlib_save_image(argv[optind+2]);
	  imlib_free_image();
	}
      else
	{
	  fprintf(stderr,_("Error loading image %s!\n"),argv[1]);
	  exit(-1);
	}
    }
  else
    {
      fprintf(stderr,_("Error loading image %s!\n"),argv[2]);
      exit(-1);
    }
  exit(0);
}
